"""
Aggregation operations for DataFrame.

This module provides aggregation operations that can be mixed into
the DataFrame class to add aggregation capabilities.
"""

from .operations import AggregationOperations

__all__ = ["AggregationOperations"]
